// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textgrammarcheck_debug.h"


Q_LOGGING_CATEGORY(TEXTGRAMMARCHECK_LOG, "org.kde.kf.textgrammarcheck", QtInfoMsg)

