/****************************************************************************
** Meta object code from reading C++ file 'plaintexteditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../ktextaddons-1.5.2/textcustomeditor/plaintexteditor/plaintexteditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plaintexteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextCustomEditor__PlainTextEditor_t {
    QByteArrayData data[22];
    char stringdata0[340];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextCustomEditor__PlainTextEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextCustomEditor__PlainTextEditor_t qt_meta_stringdata_TextCustomEditor__PlainTextEditor = {
    {
QT_MOC_LITERAL(0, 0, 33), // "TextCustomEditor::PlainTextEd..."
QT_MOC_LITERAL(1, 34, 8), // "findText"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 11), // "replaceText"
QT_MOC_LITERAL(4, 56, 23), // "spellCheckerAutoCorrect"
QT_MOC_LITERAL(5, 80, 11), // "currentWord"
QT_MOC_LITERAL(6, 92, 15), // "autoCorrectWord"
QT_MOC_LITERAL(7, 108, 20), // "checkSpellingChanged"
QT_MOC_LITERAL(8, 129, 15), // "languageChanged"
QT_MOC_LITERAL(9, 145, 16), // "spellCheckStatus"
QT_MOC_LITERAL(10, 162, 3), // "say"
QT_MOC_LITERAL(11, 166, 4), // "text"
QT_MOC_LITERAL(12, 171, 27), // "slotDisplayMessageIndicator"
QT_MOC_LITERAL(13, 199, 7), // "message"
QT_MOC_LITERAL(14, 207, 17), // "slotCheckSpelling"
QT_MOC_LITERAL(15, 225, 13), // "slotSpeakText"
QT_MOC_LITERAL(16, 239, 13), // "slotZoomReset"
QT_MOC_LITERAL(17, 253, 13), // "searchSupport"
QT_MOC_LITERAL(18, 267, 20), // "spellCheckingSupport"
QT_MOC_LITERAL(19, 288, 19), // "textToSpeechSupport"
QT_MOC_LITERAL(20, 308, 18), // "webShortcutSupport"
QT_MOC_LITERAL(21, 327, 12) // "emojiSupport"

    },
    "TextCustomEditor::PlainTextEditor\0"
    "findText\0\0replaceText\0spellCheckerAutoCorrect\0"
    "currentWord\0autoCorrectWord\0"
    "checkSpellingChanged\0languageChanged\0"
    "spellCheckStatus\0say\0text\0"
    "slotDisplayMessageIndicator\0message\0"
    "slotCheckSpelling\0slotSpeakText\0"
    "slotZoomReset\0searchSupport\0"
    "spellCheckingSupport\0textToSpeechSupport\0"
    "webShortcutSupport\0emojiSupport"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextCustomEditor__PlainTextEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       5,   94, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    0,   70,    2, 0x06 /* Public */,
       4,    2,   71,    2, 0x06 /* Public */,
       7,    1,   76,    2, 0x06 /* Public */,
       8,    1,   79,    2, 0x06 /* Public */,
       9,    1,   82,    2, 0x06 /* Public */,
      10,    1,   85,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1,   88,    2, 0x0a /* Public */,
      14,    0,   91,    2, 0x0a /* Public */,
      15,    0,   92,    2, 0x0a /* Public */,
      16,    0,   93,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,    6,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,   11,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      17, QMetaType::Bool, 0x00095103,
      18, QMetaType::Bool, 0x00095103,
      19, QMetaType::Bool, 0x00095103,
      20, QMetaType::Bool, 0x00095103,
      21, QMetaType::Bool, 0x00095103,

       0        // eod
};

void TextCustomEditor::PlainTextEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlainTextEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->findText(); break;
        case 1: _t->replaceText(); break;
        case 2: _t->spellCheckerAutoCorrect((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->checkSpellingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->languageChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->spellCheckStatus((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->say((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->slotDisplayMessageIndicator((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->slotCheckSpelling(); break;
        case 9: _t->slotSpeakText(); break;
        case 10: _t->slotZoomReset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlainTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::findText)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::replaceText)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::spellCheckerAutoCorrect)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::checkSpellingChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::languageChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::spellCheckStatus)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (PlainTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainTextEditor::say)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PlainTextEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->searchSupport(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->spellCheckingSupport(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->textToSpeechSupport(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->webShortcutSupport(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->emojiSupport(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PlainTextEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSearchSupport(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setSpellCheckingSupport(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setTextToSpeechSupport(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setWebShortcutSupport(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setEmojiSupport(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TextCustomEditor::PlainTextEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlainTextEdit::staticMetaObject>(),
    qt_meta_stringdata_TextCustomEditor__PlainTextEditor.data,
    qt_meta_data_TextCustomEditor__PlainTextEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextCustomEditor::PlainTextEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::PlainTextEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextCustomEditor__PlainTextEditor.stringdata0))
        return static_cast<void*>(this);
    return QPlainTextEdit::qt_metacast(_clname);
}

int TextCustomEditor::PlainTextEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TextCustomEditor::PlainTextEditor::findText()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::PlainTextEditor::replaceText()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextCustomEditor::PlainTextEditor::spellCheckerAutoCorrect(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TextCustomEditor::PlainTextEditor::checkSpellingChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TextCustomEditor::PlainTextEditor::languageChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TextCustomEditor::PlainTextEditor::spellCheckStatus(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TextCustomEditor::PlainTextEditor::say(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
