/*
  SPDX-FileCopyrightText: 2022-2023 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once

#include <QString>

namespace DeeplEngineUtil
{
Q_REQUIRED_RESULT QString groupName();
Q_REQUIRED_RESULT QString freeLicenseKey();
Q_REQUIRED_RESULT QString apiGroupName();
Q_REQUIRED_RESULT QString translatorGroupName();
};
