// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "lingvatranslator_debug.h"


Q_LOGGING_CATEGORY(TRANSLATOR_LINGVA_LOG, "org.kde.kf.texttranslator.lingva", QtInfoMsg)

