/****************************************************************************
** Meta object code from reading C++ file 'richtexteditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../ktextaddons-1.5.2/textcustomeditor/richtexteditor/richtexteditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'richtexteditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextCustomEditor__RichTextEditor_t {
    QByteArrayData data[26];
    char stringdata0[415];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextCustomEditor__RichTextEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextCustomEditor__RichTextEditor_t qt_meta_stringdata_TextCustomEditor__RichTextEditor = {
    {
QT_MOC_LITERAL(0, 0, 32), // "TextCustomEditor::RichTextEditor"
QT_MOC_LITERAL(1, 33, 3), // "say"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 4), // "text"
QT_MOC_LITERAL(4, 43, 8), // "findText"
QT_MOC_LITERAL(5, 52, 11), // "replaceText"
QT_MOC_LITERAL(6, 64, 23), // "spellCheckerAutoCorrect"
QT_MOC_LITERAL(7, 88, 11), // "currentWord"
QT_MOC_LITERAL(8, 100, 15), // "autoCorrectWord"
QT_MOC_LITERAL(9, 116, 20), // "checkSpellingChanged"
QT_MOC_LITERAL(10, 137, 15), // "languageChanged"
QT_MOC_LITERAL(11, 153, 16), // "spellCheckStatus"
QT_MOC_LITERAL(12, 170, 21), // "spellCheckingFinished"
QT_MOC_LITERAL(13, 192, 21), // "spellCheckingCanceled"
QT_MOC_LITERAL(14, 214, 27), // "slotDisplayMessageIndicator"
QT_MOC_LITERAL(15, 242, 7), // "message"
QT_MOC_LITERAL(16, 250, 13), // "slotSpeakText"
QT_MOC_LITERAL(17, 264, 17), // "slotCheckSpelling"
QT_MOC_LITERAL(18, 282, 13), // "slotZoomReset"
QT_MOC_LITERAL(19, 296, 13), // "searchSupport"
QT_MOC_LITERAL(20, 310, 20), // "spellCheckingSupport"
QT_MOC_LITERAL(21, 331, 19), // "textToSpeechSupport"
QT_MOC_LITERAL(22, 351, 20), // "activateLanguageMenu"
QT_MOC_LITERAL(23, 372, 10), // "tabSupport"
QT_MOC_LITERAL(24, 383, 18), // "webShortcutSupport"
QT_MOC_LITERAL(25, 402, 12) // "emojiSupport"

    },
    "TextCustomEditor::RichTextEditor\0say\0"
    "\0text\0findText\0replaceText\0"
    "spellCheckerAutoCorrect\0currentWord\0"
    "autoCorrectWord\0checkSpellingChanged\0"
    "languageChanged\0spellCheckStatus\0"
    "spellCheckingFinished\0spellCheckingCanceled\0"
    "slotDisplayMessageIndicator\0message\0"
    "slotSpeakText\0slotCheckSpelling\0"
    "slotZoomReset\0searchSupport\0"
    "spellCheckingSupport\0textToSpeechSupport\0"
    "activateLanguageMenu\0tabSupport\0"
    "webShortcutSupport\0emojiSupport"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextCustomEditor__RichTextEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       7,  106, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    0,   82,    2, 0x06 /* Public */,
       5,    0,   83,    2, 0x06 /* Public */,
       6,    2,   84,    2, 0x06 /* Public */,
       9,    1,   89,    2, 0x06 /* Public */,
      10,    1,   92,    2, 0x06 /* Public */,
      11,    1,   95,    2, 0x06 /* Public */,
      12,    0,   98,    2, 0x06 /* Public */,
      13,    0,   99,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    1,  100,    2, 0x0a /* Public */,
      16,    0,  103,    2, 0x0a /* Public */,
      17,    0,  104,    2, 0x0a /* Public */,
      18,    0,  105,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    7,    8,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      19, QMetaType::Bool, 0x00095103,
      20, QMetaType::Bool, 0x00095103,
      21, QMetaType::Bool, 0x00095103,
      22, QMetaType::Bool, 0x00095103,
      23, QMetaType::Bool, 0x00095003,
      24, QMetaType::Bool, 0x00095103,
      25, QMetaType::Bool, 0x00095103,

       0        // eod
};

void TextCustomEditor::RichTextEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RichTextEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->say((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->findText(); break;
        case 2: _t->replaceText(); break;
        case 3: _t->spellCheckerAutoCorrect((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->checkSpellingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->languageChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->spellCheckStatus((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->spellCheckingFinished(); break;
        case 8: _t->spellCheckingCanceled(); break;
        case 9: _t->slotDisplayMessageIndicator((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->slotSpeakText(); break;
        case 11: _t->slotCheckSpelling(); break;
        case 12: _t->slotZoomReset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RichTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::say)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::findText)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::replaceText)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::spellCheckerAutoCorrect)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::checkSpellingChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::languageChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::spellCheckStatus)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::spellCheckingFinished)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (RichTextEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RichTextEditor::spellCheckingCanceled)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<RichTextEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->searchSupport(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->spellCheckingSupport(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->textToSpeechSupport(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->activateLanguageMenu(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->allowTabSupport(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->webShortcutSupport(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->emojiSupport(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<RichTextEditor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSearchSupport(*reinterpret_cast< bool*>(_v)); break;
        case 1: _t->setSpellCheckingSupport(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setTextToSpeechSupport(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setActivateLanguageMenu(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setAllowTabSupport(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setWebShortcutSupport(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setEmojiSupport(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TextCustomEditor::RichTextEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QTextEdit::staticMetaObject>(),
    qt_meta_stringdata_TextCustomEditor__RichTextEditor.data,
    qt_meta_data_TextCustomEditor__RichTextEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextCustomEditor::RichTextEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::RichTextEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextCustomEditor__RichTextEditor.stringdata0))
        return static_cast<void*>(this);
    return QTextEdit::qt_metacast(_clname);
}

int TextCustomEditor::RichTextEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TextCustomEditor::RichTextEditor::say(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TextCustomEditor::RichTextEditor::findText()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextCustomEditor::RichTextEditor::replaceText()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextCustomEditor::RichTextEditor::spellCheckerAutoCorrect(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TextCustomEditor::RichTextEditor::checkSpellingChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TextCustomEditor::RichTextEditor::languageChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TextCustomEditor::RichTextEditor::spellCheckStatus(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void TextCustomEditor::RichTextEditor::spellCheckingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TextCustomEditor::RichTextEditor::spellCheckingCanceled()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
