/****************************************************************************
** Meta object code from reading C++ file 'texteditfindbarbase.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../ktextaddons-1.5.2/textcustomeditor/widgets/texteditfindbarbase.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'texteditfindbarbase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase_t {
    QByteArrayData data[17];
    char stringdata0[237];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase_t qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase = {
    {
QT_MOC_LITERAL(0, 0, 37), // "TextCustomEditor::TextEditFin..."
QT_MOC_LITERAL(1, 38, 23), // "displayMessageIndicator"
QT_MOC_LITERAL(2, 62, 0), // ""
QT_MOC_LITERAL(3, 63, 7), // "message"
QT_MOC_LITERAL(4, 71, 11), // "hideFindBar"
QT_MOC_LITERAL(5, 83, 8), // "findNext"
QT_MOC_LITERAL(6, 92, 8), // "findPrev"
QT_MOC_LITERAL(7, 101, 10), // "autoSearch"
QT_MOC_LITERAL(8, 112, 3), // "str"
QT_MOC_LITERAL(9, 116, 14), // "slotSearchText"
QT_MOC_LITERAL(10, 131, 8), // "backward"
QT_MOC_LITERAL(11, 140, 12), // "isAutoSearch"
QT_MOC_LITERAL(12, 153, 8), // "closeBar"
QT_MOC_LITERAL(13, 162, 15), // "slotClearSearch"
QT_MOC_LITERAL(14, 178, 23), // "slotUpdateSearchOptions"
QT_MOC_LITERAL(15, 202, 15), // "slotReplaceText"
QT_MOC_LITERAL(16, 218, 18) // "slotReplaceAllText"

    },
    "TextCustomEditor::TextEditFindBarBase\0"
    "displayMessageIndicator\0\0message\0"
    "hideFindBar\0findNext\0findPrev\0autoSearch\0"
    "str\0slotSearchText\0backward\0isAutoSearch\0"
    "closeBar\0slotClearSearch\0"
    "slotUpdateSearchOptions\0slotReplaceText\0"
    "slotReplaceAllText"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextCustomEditor__TextEditFindBarBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    0,   82,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   83,    2, 0x0a /* Public */,
       6,    0,   84,    2, 0x0a /* Public */,
       7,    1,   85,    2, 0x0a /* Public */,
       9,    2,   88,    2, 0x0a /* Public */,
       9,    1,   93,    2, 0x2a /* Public | MethodCloned */,
       9,    0,   96,    2, 0x2a /* Public | MethodCloned */,
      12,    0,   97,    2, 0x0a /* Public */,
      13,    0,   98,    2, 0x08 /* Private */,
      14,    0,   99,    2, 0x08 /* Private */,
      15,    0,  100,    2, 0x08 /* Private */,
      16,    0,  101,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,   10,   11,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TextCustomEditor::TextEditFindBarBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextEditFindBarBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->displayMessageIndicator((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->hideFindBar(); break;
        case 2: _t->findNext(); break;
        case 3: _t->findPrev(); break;
        case 4: _t->autoSearch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotSearchText((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->slotSearchText((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->slotSearchText(); break;
        case 8: _t->closeBar(); break;
        case 9: _t->slotClearSearch(); break;
        case 10: _t->slotUpdateSearchOptions(); break;
        case 11: _t->slotReplaceText(); break;
        case 12: _t->slotReplaceAllText(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextEditFindBarBase::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditFindBarBase::displayMessageIndicator)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextEditFindBarBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextEditFindBarBase::hideFindBar)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextCustomEditor::TextEditFindBarBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase.data,
    qt_meta_data_TextCustomEditor__TextEditFindBarBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextCustomEditor::TextEditFindBarBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::TextEditFindBarBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextCustomEditor__TextEditFindBarBase.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextCustomEditor::TextEditFindBarBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::TextEditFindBarBase::displayMessageIndicator(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TextCustomEditor::TextEditFindBarBase::hideFindBar()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
