/****************************************************************************
** Meta object code from reading C++ file 'texttospeech.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../ktextaddons-1.5.2/textedittexttospeech/texttospeech.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'texttospeech.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech_t {
    QByteArrayData data[17];
    char stringdata0[158];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech_t qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech = {
    {
QT_MOC_LITERAL(0, 0, 34), // "TextEditTextToSpeech::TextToS..."
QT_MOC_LITERAL(1, 35, 12), // "stateChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 19), // "TextToSpeech::State"
QT_MOC_LITERAL(4, 69, 3), // "say"
QT_MOC_LITERAL(5, 73, 4), // "text"
QT_MOC_LITERAL(6, 78, 4), // "stop"
QT_MOC_LITERAL(7, 83, 5), // "pause"
QT_MOC_LITERAL(8, 89, 6), // "resume"
QT_MOC_LITERAL(9, 96, 7), // "setRate"
QT_MOC_LITERAL(10, 104, 4), // "rate"
QT_MOC_LITERAL(11, 109, 8), // "setPitch"
QT_MOC_LITERAL(12, 118, 5), // "pitch"
QT_MOC_LITERAL(13, 124, 9), // "setVolume"
QT_MOC_LITERAL(14, 134, 6), // "volume"
QT_MOC_LITERAL(15, 141, 9), // "setLocale"
QT_MOC_LITERAL(16, 151, 6) // "locale"

    },
    "TextEditTextToSpeech::TextToSpeech\0"
    "stateChanged\0\0TextToSpeech::State\0say\0"
    "text\0stop\0pause\0resume\0setRate\0rate\0"
    "setPitch\0pitch\0setVolume\0volume\0"
    "setLocale\0locale"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextEditTextToSpeech__TextToSpeech[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   62,    2, 0x0a /* Public */,
       6,    0,   65,    2, 0x0a /* Public */,
       7,    0,   66,    2, 0x0a /* Public */,
       8,    0,   67,    2, 0x0a /* Public */,
       9,    1,   68,    2, 0x0a /* Public */,
      11,    1,   71,    2, 0x0a /* Public */,
      13,    1,   74,    2, 0x0a /* Public */,
      15,    1,   77,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   10,
    QMetaType::Void, QMetaType::Double,   12,
    QMetaType::Void, QMetaType::Double,   14,
    QMetaType::Void, QMetaType::QLocale,   16,

       0        // eod
};

void TextEditTextToSpeech::TextToSpeech::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextToSpeech *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast< TextToSpeech::State(*)>(_a[1]))); break;
        case 1: _t->say((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->stop(); break;
        case 3: _t->pause(); break;
        case 4: _t->resume(); break;
        case 5: _t->setRate((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 6: _t->setPitch((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 7: _t->setVolume((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 8: _t->setLocale((*reinterpret_cast< const QLocale(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextToSpeech::*)(TextToSpeech::State );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextToSpeech::stateChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextEditTextToSpeech::TextToSpeech::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech.data,
    qt_meta_data_TextEditTextToSpeech__TextToSpeech,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextEditTextToSpeech::TextToSpeech::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextEditTextToSpeech::TextToSpeech::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextEditTextToSpeech__TextToSpeech.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TextEditTextToSpeech::TextToSpeech::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void TextEditTextToSpeech::TextToSpeech::stateChanged(TextToSpeech::State _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
