/****************************************************************************
** Meta object code from reading C++ file 'importabstractautocorrection.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../ktextaddons-1.5.2/textautocorrection/core/import/importabstractautocorrection.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'importabstractautocorrection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextAutoCorrectionCore__ImportAbstractAutocorrection_t {
    QByteArrayData data[4];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextAutoCorrectionCore__ImportAbstractAutocorrection_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextAutoCorrectionCore__ImportAbstractAutocorrection_t qt_meta_stringdata_TextAutoCorrectionCore__ImportAbstractAutocorrection = {
    {
QT_MOC_LITERAL(0, 0, 52), // "TextAutoCorrectionCore::Impor..."
QT_MOC_LITERAL(1, 53, 13), // "LoadAttribute"
QT_MOC_LITERAL(2, 67, 3), // "All"
QT_MOC_LITERAL(3, 71, 11) // "SuperScript"

    },
    "TextAutoCorrectionCore::ImportAbstractAutocorrection\0"
    "LoadAttribute\0All\0SuperScript"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextAutoCorrectionCore__ImportAbstractAutocorrection[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       1,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    2,   19,

 // enum data: key, value
       2, uint(TextAutoCorrectionCore::ImportAbstractAutocorrection::All),
       3, uint(TextAutoCorrectionCore::ImportAbstractAutocorrection::SuperScript),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject TextAutoCorrectionCore::ImportAbstractAutocorrection::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_TextAutoCorrectionCore__ImportAbstractAutocorrection.data,
    qt_meta_data_TextAutoCorrectionCore__ImportAbstractAutocorrection,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
