/****************************************************************************
** Meta object code from reading C++ file 'textfindreplacewidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../ktextaddons-1.5.2/textcustomeditor/widgets/textfindreplacewidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textfindreplacewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TextCustomEditor__TextFindWidget_t {
    QByteArrayData data[8];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextCustomEditor__TextFindWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextCustomEditor__TextFindWidget_t qt_meta_stringdata_TextCustomEditor__TextFindWidget = {
    {
QT_MOC_LITERAL(0, 0, 32), // "TextCustomEditor::TextFindWidget"
QT_MOC_LITERAL(1, 33, 8), // "findNext"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 8), // "findPrev"
QT_MOC_LITERAL(4, 52, 11), // "clearSearch"
QT_MOC_LITERAL(5, 64, 10), // "autoSearch"
QT_MOC_LITERAL(6, 75, 19), // "updateSearchOptions"
QT_MOC_LITERAL(7, 95, 17) // "searchStringEmpty"

    },
    "TextCustomEditor::TextFindWidget\0"
    "findNext\0\0findPrev\0clearSearch\0"
    "autoSearch\0updateSearchOptions\0"
    "searchStringEmpty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextCustomEditor__TextFindWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,
       4,    0,   46,    2, 0x06 /* Public */,
       5,    1,   47,    2, 0x06 /* Public */,
       6,    0,   50,    2, 0x06 /* Public */,
       7,    1,   51,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void TextCustomEditor::TextFindWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextFindWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->findNext(); break;
        case 1: _t->findPrev(); break;
        case 2: _t->clearSearch(); break;
        case 3: _t->autoSearch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->updateSearchOptions(); break;
        case 5: _t->searchStringEmpty((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextFindWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::findNext)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextFindWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::findPrev)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TextFindWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::clearSearch)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TextFindWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::autoSearch)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TextFindWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::updateSearchOptions)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TextFindWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextFindWidget::searchStringEmpty)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextCustomEditor::TextFindWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TextCustomEditor__TextFindWidget.data,
    qt_meta_data_TextCustomEditor__TextFindWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextCustomEditor::TextFindWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::TextFindWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextCustomEditor__TextFindWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextCustomEditor::TextFindWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::TextFindWidget::findNext()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::TextFindWidget::findPrev()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TextCustomEditor::TextFindWidget::clearSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TextCustomEditor::TextFindWidget::autoSearch(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TextCustomEditor::TextFindWidget::updateSearchOptions()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TextCustomEditor::TextFindWidget::searchStringEmpty(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
struct qt_meta_stringdata_TextCustomEditor__TextReplaceWidget_t {
    QByteArrayData data[5];
    char stringdata0[86];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextCustomEditor__TextReplaceWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextCustomEditor__TextReplaceWidget_t qt_meta_stringdata_TextCustomEditor__TextReplaceWidget = {
    {
QT_MOC_LITERAL(0, 0, 35), // "TextCustomEditor::TextReplace..."
QT_MOC_LITERAL(1, 36, 11), // "replaceText"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 14), // "replaceAllText"
QT_MOC_LITERAL(4, 64, 21) // "slotSearchStringEmpty"

    },
    "TextCustomEditor::TextReplaceWidget\0"
    "replaceText\0\0replaceAllText\0"
    "slotSearchStringEmpty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextCustomEditor__TextReplaceWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   31,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void TextCustomEditor::TextReplaceWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TextReplaceWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->replaceText(); break;
        case 1: _t->replaceAllText(); break;
        case 2: _t->slotSearchStringEmpty((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TextReplaceWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextReplaceWidget::replaceText)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TextReplaceWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TextReplaceWidget::replaceAllText)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TextCustomEditor::TextReplaceWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TextCustomEditor__TextReplaceWidget.data,
    qt_meta_data_TextCustomEditor__TextReplaceWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextCustomEditor::TextReplaceWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextCustomEditor::TextReplaceWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextCustomEditor__TextReplaceWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TextCustomEditor::TextReplaceWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TextCustomEditor::TextReplaceWidget::replaceText()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TextCustomEditor::TextReplaceWidget::replaceAllText()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
