// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "textedittexttospeech_debug.h"


Q_LOGGING_CATEGORY(TEXTEDITTEXTTOSPEECH_LOG, "org.kde.kf.textedittexttospeech", QtInfoMsg)

